Struct EmptySubtreeRoots
pub struct EmptySubtreeRoots;Expand description
Contains precomputed roots of empty subtrees in a Merkle tree.
Implementations
impl EmptySubtreeRoots
impl EmptySubtreeRoots
pub const fn empty_hashes(tree_depth: u8) -> &'static [Word]
pub const fn empty_hashes(tree_depth: u8) -> &'static [Word]
Returns a static slice with roots of empty subtrees of a Merkle tree starting at the specified depth.