Packable

Trait Packable 

pub trait Packable:
    'static
    + Default
    + Copy
    + Send
    + Sync
    + PartialEq
    + Eq { }
A trait to constrain types that can be packed into a packed value.

The Packable trait allows us to specify implementations for potentially conflicting types.

Dyn Compatibility

This trait is not dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety", so this trait is not object safe.

Implementations on Foreign Types



impl Packable for u8



impl Packable for u16



impl Packable for u32



impl Packable for u64



impl Packable for u128



impl Packable for Felt



impl Packable for Goldilocks



impl<MP> Packable for MontyField31<MP>
where MP: MontyParameters,

Implementors



impl<F> Packable for QuinticTrinomialExtensionField<F>
where F: QuinticTrinomialExtendable,



impl<F, const D: usize> Packable for BinomialExtensionField<F, D>