prove_from_trace_sync

Function prove_from_trace_sync 

pub fn prove_from_trace_sync(
    inputs: TraceProvingInputs,
) -> Result<(StackOutputs, ExecutionProof), ExecutionError>
Builds an execution trace from pre-executed trace inputs and proves it synchronously.

This is useful when program execution has already happened elsewhere and only trace building plus proof generation remain. The execution settings are already reflected in the supplied TraceBuildInputs, so only proof-generation options remain in this API.