pub enum MerkleData {
MerkleTree(Vec<String>),
SparseMerkleTree(Vec<(u64, String)>),
PartialMerkleTree(Vec<((u8, u64), String)>),
}Expand description
Struct used to deserialize merkle data from input file. Merkle data can be represented as a merkle tree or a Sparse Merkle Tree.
Variants
MerkleTree(Vec<String>)
String representation of a merkle tree. The merkle tree is represented as a vector of 32 byte hex strings where each string represents a leaf in the tree.
SparseMerkleTree(Vec<(u64, String)>)
String representation of a Sparse Merkle Tree. The Sparse Merkle Tree is represented as a vector of tuples where each tuple consists of a u64 node index and a 32 byte hex string representing the value of the node.
PartialMerkleTree(Vec<((u8, u64), String)>)
String representation of a Partial Merkle Tree. The Partial Merkle Tree is represented as a vector of tuples where each tuple consists of a leaf index tuple (depth, index) and a 32 byte hex string representing the value of the leaf.