pub struct InputFile {
pub operand_stack: Vec<String>,
pub advice_stack: Option<Vec<String>>,
pub advice_map: Option<HashMap<String, Vec<u64>>>,
pub merkle_store: Option<Vec<MerkleData>>,
}Expand description
Input file struct that is used to deserialize input data from file. It consists of four components:
- operand_stack
- advice_stack
- advice_map
- merkle_store
Fields
operand_stack: Vec<String>String representation of the initial operand stack, composed of chained field elements.
advice_stack: Option<Vec<String>>Optional string representation of the initial advice stack, composed of chained field elements.
advice_map: Option<HashMap<String, Vec<u64>>>Optional map of 32 byte hex strings to vectors of u64s representing the initial advice map.
merkle_store: Option<Vec<MerkleData>>Optional vector of merkle data which will be loaded into the initial merkle store. Merkle data is represented as 32 byte hex strings and node indexes are represented as u64s.
Implementations
Sourceimpl InputFile
Helper methods to interact with the input file
impl InputFile
Helper methods to interact with the input file
pub fn read( inputs_path: &Option<PathBuf>, program_path: &Path, ) -> Result<Self, Report>
Sourcepub fn parse_advice_inputs(&self) -> Result<AdviceInputs, String>
pub fn parse_advice_inputs(&self) -> Result<AdviceInputs, String>
Parse advice inputs from the input file.
Sourcepub fn parse_stack_inputs(&self) -> Result<StackInputs, String>
pub fn parse_stack_inputs(&self) -> Result<StackInputs, String>
Parse and return the stack inputs for the program.