pub struct DefaultHost<D = DefaultDebugHandler, S = DefaultSourceManager>where
D: DebugHandler,
S: SourceManager,{ /* private fields */ }Expand description
A default SyncHost implementation that provides the essential functionality required by the VM.
Implementations
Sourceimpl<D, S> DefaultHost<D, S>where
D: DebugHandler,
S: SourceManager,
impl<D, S> DefaultHost<D, S>where
D: DebugHandler,
S: SourceManager,
Sourcepub fn with_source_manager<O>(self, source_manager: Arc<O>) -> DefaultHost<D, O>where
O: SourceManager,
pub fn with_source_manager<O>(self, source_manager: Arc<O>) -> DefaultHost<D, O>where
O: SourceManager,
Use the given source manager implementation instead of the default one
[DefaultSourceManager].
Sourcepub fn load_library(
&mut self,
library: impl Into<HostLibrary>,
) -> Result<(), ExecutionError>
pub fn load_library( &mut self, library: impl Into<HostLibrary>, ) -> Result<(), ExecutionError>
Loads a HostLibrary containing a [MastForest] with its list of event handlers.
Sourcepub fn with_library(
self,
library: impl Into<HostLibrary>,
) -> Result<DefaultHost<D, S>, ExecutionError>
pub fn with_library( self, library: impl Into<HostLibrary>, ) -> Result<DefaultHost<D, S>, ExecutionError>
Adds a HostLibrary containing a [MastForest] with its list of event handlers.
to the host.
Sourcepub fn register_handler(
&mut self,
event: EventName,
handler: Arc<dyn EventHandler>,
) -> Result<(), ExecutionError>
pub fn register_handler( &mut self, event: EventName, handler: Arc<dyn EventHandler>, ) -> Result<(), ExecutionError>
Registers a single EventHandler into this host.
The handler can be either a closure or a free function with signature
fn(&mut ProcessorState) -> Result<(), EventHandler>
Sourcepub fn unregister_handler(&mut self, id: EventId) -> bool
pub fn unregister_handler(&mut self, id: EventId) -> bool
Un-registers a handler with the given id, returning a flag indicating whether a handler was previously registered with this id.
Sourcepub fn replace_handler(
&mut self,
event: EventName,
handler: Arc<dyn EventHandler>,
) -> bool
pub fn replace_handler( &mut self, event: EventName, handler: Arc<dyn EventHandler>, ) -> bool
Replaces a handler with the given event, returning a flag indicating whether a handler was previously registered with this event ID.
Sourcepub fn with_debug_handler<H>(self, handler: H) -> DefaultHost<H, S>where
H: DebugHandler,
pub fn with_debug_handler<H>(self, handler: H) -> DefaultHost<H, S>where
H: DebugHandler,
Replace the current DebugHandler with a custom one.
Sourcepub fn debug_handler(&self) -> &D
pub fn debug_handler(&self) -> &D
Returns a reference to the DebugHandler, useful for recovering debug information
emitted during a program execution.