Struct Kernel
pub struct Kernel(/* private fields */);Expand description
A list of exported kernel procedure hashes defining a VM kernel.
The internally-stored list always has a consistent order, regardless of the order of procedure list used to instantiate a kernel.
Implementations
impl Kernel
impl Kernel
pub const MAX_NUM_PROCEDURES: usize
pub const MAX_NUM_PROCEDURES: usize
The maximum number of procedures which can be exported from a Kernel.
pub fn new(proc_hashes: &[Word]) -> Result<Kernel, KernelError>
pub fn new(proc_hashes: &[Word]) -> Result<Kernel, KernelError>
Returns a new Kernel instantiated with the specified procedure hashes.
Hashes are canonicalized into a consistent internal order.
Errors
Returns an error if:
proc_hashescontains duplicates.proc_hashes.len()exceedsMAX_NUM_PROCEDURES.
pub fn from_hashes(hashes: Vec<Word>) -> Result<Kernel, KernelError>
pub fn from_hashes(hashes: Vec<Word>) -> Result<Kernel, KernelError>
Returns a new Kernel from owned procedure hashes.
Hashes are canonicalized into a consistent internal order.
Errors
Returns an error if:
hashescontains duplicates.hashes.len()exceedsMAX_NUM_PROCEDURES.
pub fn contains_proc(&self, proc_hash: Word) -> bool
pub fn contains_proc(&self, proc_hash: Word) -> bool
Returns true if a procedure with the specified hash belongs to this kernel.
Note: the kernel is constructed from exported kernel procedures only.
pub fn proc_hashes(&self) -> &[Word]
pub fn proc_hashes(&self) -> &[Word]
Returns a list of procedure hashes contained in this kernel.