pub struct TraceBuildInputs { /* private fields */ }Expand description
Inputs required to build an execution trace from pre-executed data.
Implementations
Sourceimpl TraceBuildInputs
impl TraceBuildInputs
Sourcepub fn stack_outputs(&self) -> &StackOutputs
pub fn stack_outputs(&self) -> &StackOutputs
Returns the stack outputs captured for the execution being replayed.
Sourcepub fn precompile_requests(&self) -> &[PrecompileRequest]
pub fn precompile_requests(&self) -> &[PrecompileRequest]
Returns deferred precompile requests generated during execution.
Sourcepub fn final_precompile_transcript(&self) -> &PrecompileTranscript
pub fn final_precompile_transcript(&self) -> &PrecompileTranscript
Returns the final precompile transcript observed during execution.
Sourcepub fn precompile_transcript_digest(&self) -> Word
pub fn precompile_transcript_digest(&self) -> Word
Returns the digest of the final precompile transcript observed during execution.
Sourcepub fn program_info(&self) -> &ProgramInfo
pub fn program_info(&self) -> &ProgramInfo
Returns the program info captured for the execution being replayed.
Sourcepub fn trace_generation_context(&self) -> &TraceGenerationContext
pub fn trace_generation_context(&self) -> &TraceGenerationContext
Returns the trace replay context captured during execution.